function [AvgPower, Threshold] = RayleighCapacityPower(Rate, MeanChannel)

% Compute average transmit power to reach average rate of Rate over
% a Rayleigh fading channel with water-filling power allocation

% First, use the secant method to find the correct threshold given
% mean channel and required rate
thhypo = 0;
rate = ratehelper(thhypo,MeanChannel);
err = rate-Rate;
step = 1;

% find correct water level
while abs(err) > 0.00001
    thhypo = thhypo + step*sign(err);
    rate = ratehelper(thhypo,MeanChannel);
    errprev = err;
    err = rate - Rate;
    if err*errprev < 0
        step = 0.5*step;
    end
end

% Find the average power corresponding to the Threshold
Threshold = thhypo;
AvgPower = exp(-Threshold/MeanChannel)/Threshold -...
    expint(Threshold/MeanChannel)/MeanChannel;
